"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _lodash = require("lodash");
var _constants = require("./constants");
var _helpers = require("../helpers");
var _default = (
// eslint-disable-next-line
state = {}, {
  type,
  data,
  err,
  key,
  pluralized
}) => {
  if (!key) {
    throw new Error('Key required for managing state');
  }
  const resource = key;
  const resources = pluralized;
  const handlers = {
    [_constants.FETCHING]() {
      return {
        fetching: true
      };
    },
    [_constants.FETCHED]: () => ({
      ...state,
      ...data,
      fetchingError: !(0, _helpers.isEmpty)(err),
      fetching: false,
      err
    }),
    [_constants.UPDATED]() {
      const copiedState = {
        ...state
      };
      const multiOp = (0, _lodash.get)(data, resources, []);
      const singleOp = (0, _lodash.get)(data, resource, {});
      if (resource in copiedState) copiedState[resource] = singleOp;
      if (resources in copiedState) {
        if (!Array.isArray(copiedState[resources])) copiedState[resources] = [];
        const updateCopiedStateResources = xs => {
          // ensures each is a document
          if (!(0, _helpers.hasId)(xs)) return;
          const match = copiedState[resources].find(item => (0, _helpers.hasId)(item) && item.id === xs.id);
          if (match) {
            Object.assign(match, xs);
          } else {
            copiedState[resources].push(xs);
          }
        };
        (0, _lodash.forEach)(multiOp, updateCopiedStateResources);
        updateCopiedStateResources(singleOp);
      }
      return {
        ...data,
        ...copiedState
      };
    },
    [_constants.CREATED]() {
      if ((0, _helpers.isEmpty)(data)) return state;
      const next = (0, _lodash.get)(data, resource, {});
      const prev = {
        ...state
      };
      if (resource in prev) {
        prev[resource] = next;
      } else {
        prev[resources] = Array.isArray(state[resources]) ? [next, ...state[resources]] : [next];
      }
      return prev;
    },
    /**
     * @TODO:
     * Find better solution.
     */
    [_constants.DELETED]() {
      return (0, _helpers.isEmpty)(data) ? state : {
        ...state,
        [resource]: Array.isArray(state[resource]) ? state[resource].filter(({
          id
        }) => id !== data.id) : state[resource],
        [resources]: Array.isArray(state[resources]) ? state[resources].filter(({
          id
        }) => id !== data.id) : state[resources]
      };
    },
    /**
     * @TODO:
     * Find better solution.
     */
    [_constants.DELETED_MANY]() {
      return (0, _helpers.isEmpty)(data) ? state : {
        ...state,
        [resource]: Array.isArray(state[resource]) ? state[resource].filter(({
          id
        }) => !data.ids.includes(id)) : null,
        [resources]: Array.isArray(state[resources]) ? state[resources].filter(({
          id
        }) => !data.ids.includes(id)) : null
      };
    }
  };
  return (0, _helpers.getFn)(handlers, type);
};
exports.default = _default;